import React from 'react';
import { resolveJSXElement, filterRepsonsiveSize, filterThemeName, rclassnames, renderJSXFunc, tryUseContext } from '@reboot-ui/common';
import Anchor from '@reboot-ui/icomponent-anchor';
import CollapseProto from '@reboot-ui/icomponent-collapse';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

var ControlledCollapse = React.forwardRef(CollapseProto);
var NavbarContext = React.createContext({});
var ctxSymbol = Symbol('#Navbar');
var useHookSymbols = {
  collapse: Symbol('#Navbar#useHook#collapse')
};

var useInitContextState = function useInitContextState(SYMBOL) {
  if (SYMBOL === void 0) {
    SYMBOL = ctxSymbol;
  }

  var _a = React.useState(true),
      collapse = _a[0],
      setCollapse = _a[1];

  var context = {
    symbol: SYMBOL,
    collapse: collapse,
    refs: {},
    _toggleCollapse: function _toggleCollapse(nextStatus) {
      if (nextStatus === void 0) {
        nextStatus = !context.collapse;
      }

      return setCollapse(nextStatus);
    },
    _registerRef: function _registerRef(member, ref) {
      switch (member) {
        case 'toggle':
        case 'collapse':
          if (context.refs[member]) throw new Error("[Navbar] member ref has been existed, stop add member under this context!");
          context.refs[member] = ref;
          break;

        default:
          throw new Error("[Navbar] unknown member ref '" + member + "', report to your administrator.");
      }
    },
    _unregisterRef: function _unregisterRef(member) {
      if (context.refs[member]) context.refs[member] = null;
    }
  }; // repass change of `collapse` state to last context

  var lastCtxState = tryUseContext(NavbarContext);
  React.useEffect(function () {
    if (lastCtxState.symbol === useHookSymbols.collapse) {
      lastCtxState._toggleCollapse(context.collapse);
    }
  }, [context.collapse]);
  return [context];
};

var useRefRegistry = function useRefRegistry(memberType, ref) {
  var navbarCtx = tryUseContext(NavbarContext);
  React.useEffect(function () {
    if (navbarCtx.symbol !== ctxSymbol) return;

    navbarCtx._registerRef(memberType, ref);

    return function () {
      navbarCtx._unregisterRef(memberType);
    };
  }, []);
  return [navbarCtx, ref];
};

var NavbarProto = React.forwardRef(function (_a, ref) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'nav' : _b,
      expandWhen = _a.expandWhen,
      theme = _a.theme,
      bgTheme = _a.bgTheme,
      props = __rest(_a, ["children", "as", "expandWhen", "theme", "bgTheme"]);

  var JSXEl = resolveJSXElement(_as, {
    allowedHTMLTags: ['div', 'nav', 'header']
  });
  expandWhen = filterRepsonsiveSize(expandWhen);
  theme = filterThemeName(theme);
  bgTheme = filterThemeName(bgTheme);
  var contextState = useInitContextState(ctxSymbol)[0];
  return React.createElement(NavbarContext.Provider, {
    value: contextState
  }, React.createElement(JSXEl, _extends({}, props, {
    ref: ref,
    className: rclassnames(props, ["navbar", expandWhen && "navbar-expand-" + expandWhen, theme && "navbar-" + theme, bgTheme && "bg-" + bgTheme])
  }), children));
});
/**
 * @see https://getbootstrap.com/docs/4.4/components/navbar/#supported-content
 *
 * @inner-content `.navbar-brand`
 * @inner-content `.navbar-nav`
 * @inner-content `.navbar-toggler`
 *
 * @inner-content `.navbar-text`
 * @inner-content `.collapse.navbar-collapse`
 */

var Navbar = function Navbar(props) {
  return React.createElement(NavbarProto, props);
};

Navbar.Brand = function (_a) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'a' : _b,
      props = __rest(_a, ["children", "as"]);

  if (_as === 'a') _as = Anchor;
  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  return React.createElement(JSXEl, _extends({}, props, {
    className: rclassnames(props, ['navbar-brand'])
  }), children);
};

Navbar.Text = function (_a) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'span' : _b,
      props = __rest(_a, ["children", "as"]);

  if (_as === 'a') _as = Anchor;
  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  return React.createElement(JSXEl, _extends({}, props, {
    className: rclassnames(props, ['navbar-text'])
  }), children);
};

Navbar.Toggle = React.forwardRef(function (_a, ref) {
  var // children = (<span className="navbar-toggler-icon" />),
  _b = _a.as,
      // children = (<span className="navbar-toggler-icon" />),
  _as = _b === void 0 ? 'button' : _b,
      _c = _a.target,
      target = _c === void 0 ? '' : _c,
      props = __rest(_a, ["as", "target"]);

  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  ref = ref || React.useRef(null);
  var navbarCtx = useRefRegistry('toggle', ref)[0];
  return React.createElement(JSXEl, _extends({}, props, {
    ref: ref,
    className: rclassnames(props, ['navbar-toggler']),
    type: "button",
    "data-toggle": "collapse"
  }, target && {
    'data-target': "#" + target,
    'aria-controls': "" + target
  }, {
    "aria-expanded": !navbarCtx.collapse,
    "aria-label": "Toggle navigation",
    onClick: function onClick(evt) {
      if (navbarCtx.symbol === ctxSymbol) navbarCtx._toggleCollapse(!navbarCtx.collapse);
      if (typeof props.onClick === 'function') props.onClick(evt);
    }
  }), React.createElement("span", {
    className: "navbar-toggler-icon"
  }));
});
Navbar.Collapse = React.forwardRef(function (_a, ref) {
  var children = _a.children,
      props = __rest(_a, ["children"]);

  ref = ref || React.useRef(null);
  var navbarCtx = useRefRegistry('collapse', ref)[0];
  return React.createElement(ControlledCollapse, _extends({}, props, {
    ref: ref,
    collapse: navbarCtx.collapse,
    className: rclassnames(props, ['navbar-collapse'])
  }), children);
});

Navbar.useCollapse = function (fnOrComponent) {
  var contextState = useInitContextState(useHookSymbols.collapse)[0];
  contextState.symbol = useHookSymbols.collapse;
  return React.createElement(NavbarContext.Provider, {
    value: contextState
  }, renderJSXFunc(fnOrComponent, {
    collapse: contextState.collapse
  }));
};

export default Navbar;
