'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var reactTransitionGroup = require('react-transition-group');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

var breakpoints = [
	{
		breakpoint: "xs",
		abbr: "",
		name: "Extra small",
		"min-width": "0px",
		container: ""
	},
	{
		breakpoint: "sm",
		abbr: "-sm",
		name: "Small",
		"min-width": "576px",
		container: "540px"
	},
	{
		breakpoint: "md",
		abbr: "-md",
		name: "Medium",
		"min-width": "768px",
		container: "720px"
	},
	{
		breakpoint: "lg",
		abbr: "-lg",
		name: "Large",
		"min-width": "992px",
		container: "960px"
	},
	{
		breakpoint: "xl",
		abbr: "-xl",
		name: "Extra large",
		"min-width": "1200px",
		container: "1140px"
	}
];
var colors = [
	{
		name: "blue",
		hex: "#007bff"
	},
	{
		name: "indigo",
		hex: "#6610f2"
	},
	{
		name: "purple",
		hex: "#6f42c1"
	},
	{
		name: "pink",
		hex: "#e83e8c"
	},
	{
		name: "red",
		hex: "#dc3545"
	},
	{
		name: "orange",
		hex: "#fd7e14"
	},
	{
		name: "yellow",
		hex: "#ffc107"
	},
	{
		name: "green",
		hex: "#28a745"
	},
	{
		name: "teal",
		hex: "#20c997"
	},
	{
		name: "cyan",
		hex: "#17a2b8"
	},
	{
		name: "white",
		hex: "#fff"
	},
	{
		name: "gray",
		hex: "#868e96"
	},
	{
		name: "gray-dark",
		hex: "#343a40"
	}
];
var json = {
	breakpoints: breakpoints,
	colors: colors,
	"theme-colors": [
	{
		name: "primary",
		hex: "#007bff"
	},
	{
		name: "secondary",
		hex: "#868e96"
	},
	{
		name: "success",
		hex: "#28a745"
	},
	{
		name: "danger",
		hex: "#dc3545"
	},
	{
		name: "warning",
		hex: "#ffc107"
	},
	{
		name: "info",
		hex: "#17a2b8"
	},
	{
		name: "light",
		hex: "#f8f9fa"
	},
	{
		name: "dark",
		hex: "#343a40"
	}
]
};

function arraify(value) {
  return Array.isArray(value) ? value : [value];
}
function dedupe(list) {
  if (!Array.isArray(list)) throw new Error("muse be list!");
  return Array.from(new Set(list));
}
function flatten(listOrItem) {
  var newList = [];

  if (Array.isArray(listOrItem)) {
    listOrItem.forEach(function (item) {
      newList = newList.concat(flatten(item));
    });
  } else newList.push(listOrItem);

  return newList;
}

function rclassnames(props) {
  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  var clsList = dedupe(flatten(args).concat([props.className, props["class"]]).filter(function (x) {
    return x && typeof x === 'string';
  }).map(function (x) {
    return x && x.trim();
  }).join(' ').split(' '));
  if (!clsList.length) return;
  return clsList.join(' ');
}
function tryUseContext(ContextRef) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$fallbackValue = _ref.fallbackValue,
      fallbackValue = _ref$fallbackValue === void 0 ? {} : _ref$fallbackValue;

  var ctxValue = fallbackValue;

  try {
    ctxValue = React.useContext(ContextRef);
  } catch (error) {}

  return ctxValue;
}
function renderJSXFunc(children) {
  var renderContext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (typeof children === 'function') return children.call(null, renderContext);
  return children;
}

var themes = json['theme-colors'].map(function (theme) {
  return theme.name;
});
var sizes = json['breakpoints'].map(function (x) {
  return x.breakpoint;
}).filter(function (x) {
  return x;
});
function filterThemeName() {
  var theme = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  if (themes.includes(theme)) return theme;
}
function filterRepsonsiveSize() {
  var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  if (sizes.includes(size)) return size;
}
function resolveJSXElement(inputJSXElement) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$default = _ref["default"],
      defaultValue = _ref$default === void 0 ? React.Fragment : _ref$default,
      _ref$allowedHTMLTags = _ref.allowedHTMLTags,
      allowedHTMLTags = _ref$allowedHTMLTags === void 0 ? undefined : _ref$allowedHTMLTags;

  if (!inputJSXElement) return defaultValue;

  if (allowedHTMLTags && typeof inputJSXElement === 'string') {
    inputJSXElement = inputJSXElement.toString();
    allowedHTMLTags = arraify(allowedHTMLTags).filter(function (x) {
      return typeof x === 'string';
    }).map(function (x) {
      return x.toString();
    });
    if (allowedHTMLTags.length && !allowedHTMLTags.includes(inputJSXElement)) throw new Error("[resolveJSXElement] inputJSXElement must be valid string: ".concat(allowedHTMLTags.join(', '), "; but ").concat(inputJSXElement, " given!"));
  }

  return inputJSXElement;
}

function _extends$1() {
  _extends$1 = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends$1.apply(this, arguments);
}
function __rest$1(s, e) {
  var t = {};
  for (var p in s) {
    if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  }
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
var Anchor = React.forwardRef(function (_a,
ref) {
  var children = _a.children,
      _b = _a.href,
      href = _b === void 0 ? 'javascript:;' : _b,
      props = __rest$1(_a, ["children", "href"]);
  return React.createElement("a", _extends$1({}, props, {
    ref: ref
  }, href && {
    href: href
  }), children);
});

var breakpoints$1 = [{
  breakpoint: "xs",
  abbr: "",
  name: "Extra small",
  "min-width": "0px",
  container: ""
}, {
  breakpoint: "sm",
  abbr: "-sm",
  name: "Small",
  "min-width": "576px",
  container: "540px"
}, {
  breakpoint: "md",
  abbr: "-md",
  name: "Medium",
  "min-width": "768px",
  container: "720px"
}, {
  breakpoint: "lg",
  abbr: "-lg",
  name: "Large",
  "min-width": "992px",
  container: "960px"
}, {
  breakpoint: "xl",
  abbr: "-xl",
  name: "Extra large",
  "min-width": "1200px",
  container: "1140px"
}];
var colors$1 = [{
  name: "blue",
  hex: "#007bff"
}, {
  name: "indigo",
  hex: "#6610f2"
}, {
  name: "purple",
  hex: "#6f42c1"
}, {
  name: "pink",
  hex: "#e83e8c"
}, {
  name: "red",
  hex: "#dc3545"
}, {
  name: "orange",
  hex: "#fd7e14"
}, {
  name: "yellow",
  hex: "#ffc107"
}, {
  name: "green",
  hex: "#28a745"
}, {
  name: "teal",
  hex: "#20c997"
}, {
  name: "cyan",
  hex: "#17a2b8"
}, {
  name: "white",
  hex: "#fff"
}, {
  name: "gray",
  hex: "#868e96"
}, {
  name: "gray-dark",
  hex: "#343a40"
}];
var json$1 = {
  breakpoints: breakpoints$1,
  colors: colors$1,
  "theme-colors": [{
    name: "primary",
    hex: "#007bff"
  }, {
    name: "secondary",
    hex: "#868e96"
  }, {
    name: "success",
    hex: "#28a745"
  }, {
    name: "danger",
    hex: "#dc3545"
  }, {
    name: "warning",
    hex: "#ffc107"
  }, {
    name: "info",
    hex: "#17a2b8"
  }, {
    name: "light",
    hex: "#f8f9fa"
  }, {
    name: "dark",
    hex: "#343a40"
  }]
};
function arraify$1(value) {
  return Array.isArray(value) ? value : [value];
}
function dedupe$1(list) {
  if (!Array.isArray(list)) throw new Error("muse be list!");
  return Array.from(new Set(list));
}
function flatten$1(listOrItem) {
  var newList = [];
  if (Array.isArray(listOrItem)) {
    listOrItem.forEach(function (item) {
      newList = newList.concat(flatten$1(item));
    });
  } else newList.push(listOrItem);
  return newList;
}
function rclassnames$1(props) {
  var args = [];
  for (var _i = 1; _i < arguments.length; _i++) {
    args[_i - 1] = arguments[_i];
  }
  var clsList = dedupe$1(flatten$1(args).concat([props.className, props["class"]]).filter(function (x) {
    return x && typeof x === 'string';
  }).map(function (x) {
    return x && x.trim();
  }).join(' ').split(' '));
  if (!clsList.length) return;
  return clsList.join(' ');
}
function noop$4() {
  var args = [];
  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }
}
function useDefaultValue(defaultValue, calback) {
  if (calback === void 0) {
    calback = noop$4;
  }
  var firstValRef = React.useRef(defaultValue);
  React.useEffect(function () {
    calback(firstValRef.current);
  }, [firstValRef.current]);
}
var themes$1 = json$1['theme-colors'].map(function (theme) {
  return theme.name;
});
var sizes$1 = json$1['breakpoints'].map(function (x) {
  return x.breakpoint;
}).filter(function (x) {
  return x;
});
var TransitionTimeouts = {
  Fade: 150,
  Collapse: 350,
  Modal: 300,
  Carousel: 600
};
function resolveJSXElement$1(inputJSXElement, _a) {
  var _b = _a === void 0 ? {} : _a,
      _c = _b["default"],
      defaultValue = _c === void 0 ? React.Fragment : _c,
      _d = _b.allowedHTMLTags,
      allowedHTMLTags = _d === void 0 ? undefined : _d;
  if (!inputJSXElement) return defaultValue;
  if (allowedHTMLTags && typeof inputJSXElement === 'string') {
    inputJSXElement = inputJSXElement.toString();
    allowedHTMLTags = arraify$1(allowedHTMLTags).filter(function (x) {
      return typeof x === 'string';
    }).map(function (x) {
      return x.toString();
    });
    if (allowedHTMLTags.length && !allowedHTMLTags.includes(inputJSXElement)) throw new Error("[resolveJSXElement] inputJSXElement must be valid string: " + allowedHTMLTags.join(', ') + "; but " + inputJSXElement + " given!");
  }
  return inputJSXElement;
}

function _extends$2() {
  _extends$2 = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends$2.apply(this, arguments);
}
var _assign = function __assign() {
  _assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }
    return t;
  };
  return _assign.apply(this, arguments);
};
function __rest$2(s, e) {
  var t = {};
  for (var p in s) {
    if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  }
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
var _a;
function getTransitionClass(status) {
  return transtionClasses[status] || 'collapse';
}
function getNodeHeight(node) {
  return node.scrollHeight;
}
function noop() {}
var transtionClasses = (_a = {}, _a["entering"] = 'collapsing', _a["entered"] = 'collapse show', _a["exiting"] = 'collapsing', _a["exited"] = 'collapse', _a["unmounted"] = '', _a);
function CollapseProto(_a, ref) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'div' : _b,
      _c = _a.collapse,
      propCollapsed = _c === void 0 ? true : _c,
      _d = _a.onEntering,
      onEntering = _d === void 0 ? noop : _d,
      _e = _a.onEntered,
      onEntered = _e === void 0 ? noop : _e,
      _f = _a.onExit,
      onExit = _f === void 0 ? noop : _f,
      _g = _a.onExiting,
      onExiting = _g === void 0 ? noop : _g,
      _h = _a.onExited,
      onExited = _h === void 0 ? noop : _h,
      props = __rest$2(_a, ["children", "as", "collapse", "onEntering", "onEntered", "onExit", "onExiting", "onExited"]);
  var JSXEl = resolveJSXElement$1(_as, {
  });
  var _j = React.useState(true),
      collapse = _j[0],
      setCollapse = _j[1];
  useDefaultValue(propCollapsed, function (defaultValue) {
    if (defaultValue !== collapse) setCollapse(defaultValue);
  });
  React.useEffect(function () {
    setCollapse(propCollapsed);
  }, [propCollapsed]);
  var _k = React.useState(null),
      height = _k[0],
      setHeight = _k[1];
  var hRef = React.useRef(height);
  var _onEntering = function _onEntering(node, isAppearing) {
    setHeight(getNodeHeight(node));
    onEntering(node, isAppearing);
  };
  var _onEntered = function _onEntered(node, isAppearing) {
    setHeight(null);
    hRef.current = height;
    onEntered(node, isAppearing);
  };
  var _onExit = function _onExit(node) {
    setHeight(getNodeHeight(node));
    onExit(node);
  };
  var _onExiting = function _onExiting(node) {
    hRef.current = null;
    setHeight(0);
    onExiting(node);
  };
  var _onExited = function _onExited(node) {
    setHeight(null);
    onExited(node);
  };
  ref = ref || React.useRef(null);
  return React.createElement(React.Fragment, null, React.createElement(reactTransitionGroup.Transition, _extends$2({}, reactTransitionGroup.Transition.defaultProps, {
    appear: false,
    enter: true,
    exit: true,
    timeout: {
      enter: TransitionTimeouts.Collapse,
      exit: TransitionTimeouts.Collapse
    },
    "in": !collapse,
    onEntering: _onEntering,
    onEntered: _onEntered,
    onExit: _onExit,
    onExiting: _onExiting,
    onExited: _onExited
  }), function (state) {
    return React.createElement(JSXEl, _extends$2({}, props, ref && {
      ref: ref
    }, {
      className: rclassnames$1(props, [getTransitionClass(state)]),
      style: _assign(_assign(_assign({}, props.style), height !== null && {
        height: height
      }), hRef.current !== null && {
        height: hRef.current
      })
    }), children);
  }));
}

var ControlledCollapse = React.forwardRef(CollapseProto);
var NavbarContext = React.createContext({});
var ctxSymbol = Symbol('#Navbar');
var useHookSymbols = {
  collapse: Symbol('#Navbar#useHook#collapse')
};

var useInitContextState = function useInitContextState(SYMBOL) {
  if (SYMBOL === void 0) {
    SYMBOL = ctxSymbol;
  }

  var _a = React.useState(true),
      collapse = _a[0],
      setCollapse = _a[1];

  var context = {
    symbol: SYMBOL,
    collapse: collapse,
    refs: {},
    _toggleCollapse: function _toggleCollapse(nextStatus) {
      if (nextStatus === void 0) {
        nextStatus = !context.collapse;
      }

      return setCollapse(nextStatus);
    },
    _registerRef: function _registerRef(member, ref) {
      switch (member) {
        case 'toggle':
        case 'collapse':
          if (context.refs[member]) throw new Error("[Navbar] member ref has been existed, stop add member under this context!");
          context.refs[member] = ref;
          break;

        default:
          throw new Error("[Navbar] unknown member ref '" + member + "', report to your administrator.");
      }
    },
    _unregisterRef: function _unregisterRef(member) {
      if (context.refs[member]) context.refs[member] = null;
    }
  }; // repass change of `collapse` state to last context

  var lastCtxState = tryUseContext(NavbarContext);
  React.useEffect(function () {
    if (lastCtxState.symbol === useHookSymbols.collapse) {
      lastCtxState._toggleCollapse(context.collapse);
    }
  }, [context.collapse]);
  return [context];
};

var useRefRegistry = function useRefRegistry(memberType, ref) {
  var navbarCtx = tryUseContext(NavbarContext);
  React.useEffect(function () {
    if (navbarCtx.symbol !== ctxSymbol) return;

    navbarCtx._registerRef(memberType, ref);

    return function () {
      navbarCtx._unregisterRef(memberType);
    };
  }, []);
  return [navbarCtx, ref];
};

var NavbarProto = React.forwardRef(function (_a, ref) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'nav' : _b,
      expandWhen = _a.expandWhen,
      theme = _a.theme,
      bgTheme = _a.bgTheme,
      props = __rest(_a, ["children", "as", "expandWhen", "theme", "bgTheme"]);

  var JSXEl = resolveJSXElement(_as, {
    allowedHTMLTags: ['div', 'nav', 'header']
  });
  expandWhen = filterRepsonsiveSize(expandWhen);
  theme = filterThemeName(theme);
  bgTheme = filterThemeName(bgTheme);
  var contextState = useInitContextState(ctxSymbol)[0];
  return React.createElement(NavbarContext.Provider, {
    value: contextState
  }, React.createElement(JSXEl, _extends({}, props, {
    ref: ref,
    className: rclassnames(props, ["navbar", expandWhen && "navbar-expand-" + expandWhen, theme && "navbar-" + theme, bgTheme && "bg-" + bgTheme])
  }), children));
});
/**
 * @see https://getbootstrap.com/docs/4.4/components/navbar/#supported-content
 *
 * @inner-content `.navbar-brand`
 * @inner-content `.navbar-nav`
 * @inner-content `.navbar-toggler`
 *
 * @inner-content `.navbar-text`
 * @inner-content `.collapse.navbar-collapse`
 */

var Navbar = function Navbar(props) {
  return React.createElement(NavbarProto, props);
};

Navbar.Brand = function (_a) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'a' : _b,
      props = __rest(_a, ["children", "as"]);

  if (_as === 'a') _as = Anchor;
  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  return React.createElement(JSXEl, _extends({}, props, {
    className: rclassnames(props, ['navbar-brand'])
  }), children);
};

Navbar.Text = function (_a) {
  var children = _a.children,
      _b = _a.as,
      _as = _b === void 0 ? 'span' : _b,
      props = __rest(_a, ["children", "as"]);

  if (_as === 'a') _as = Anchor;
  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  return React.createElement(JSXEl, _extends({}, props, {
    className: rclassnames(props, ['navbar-text'])
  }), children);
};

Navbar.Toggle = React.forwardRef(function (_a, ref) {
  var // children = (<span className="navbar-toggler-icon" />),
  _b = _a.as,
      // children = (<span className="navbar-toggler-icon" />),
  _as = _b === void 0 ? 'button' : _b,
      _c = _a.target,
      target = _c === void 0 ? '' : _c,
      props = __rest(_a, ["as", "target"]);

  var JSXEl = resolveJSXElement(_as, {
    /* allowedHTMLTags: [] */
  });
  ref = ref || React.useRef(null);
  var navbarCtx = useRefRegistry('toggle', ref)[0];
  return React.createElement(JSXEl, _extends({}, props, {
    ref: ref,
    className: rclassnames(props, ['navbar-toggler']),
    type: "button",
    "data-toggle": "collapse"
  }, target && {
    'data-target': "#" + target,
    'aria-controls': "" + target
  }, {
    "aria-expanded": !navbarCtx.collapse,
    "aria-label": "Toggle navigation",
    onClick: function onClick(evt) {
      if (navbarCtx.symbol === ctxSymbol) navbarCtx._toggleCollapse(!navbarCtx.collapse);
      if (typeof props.onClick === 'function') props.onClick(evt);
    }
  }), React.createElement("span", {
    className: "navbar-toggler-icon"
  }));
});
Navbar.Collapse = React.forwardRef(function (_a, ref) {
  var children = _a.children,
      props = __rest(_a, ["children"]);

  ref = ref || React.useRef(null);
  var navbarCtx = useRefRegistry('collapse', ref)[0];
  return React.createElement(ControlledCollapse, _extends({}, props, {
    ref: ref,
    collapse: navbarCtx.collapse,
    className: rclassnames(props, ['navbar-collapse'])
  }), children);
});

Navbar.useCollapse = function (fnOrComponent) {
  var contextState = useInitContextState(useHookSymbols.collapse)[0];
  contextState.symbol = useHookSymbols.collapse;
  return React.createElement(NavbarContext.Provider, {
    value: contextState
  }, renderJSXFunc(fnOrComponent, {
    collapse: contextState.collapse
  }));
};

module.exports = Navbar;
